@@ -209,11 +209,17 @@ geom_ptr geometry_builder::create_simple_poly(GeometryFactory &gf,
     std::unique_ptr<std::vector<Geometry *> > empty(new std::vector<Geometry *>);
     geom_ptr geom(gf.createPolygon(shell.release(), empty.release()));
 
+    if (geom->isEmpty()) {
+        throw std::runtime_error("Excluding empty polygon.");
+    }
     if (!geom->isValid()) {
         if (excludepoly) {
             throw std::runtime_error("Excluding broken polygon.");
         } else {
             geom = geom_ptr(geom->buffer(0));
+            if (geom->isEmpty() || !geom->isValid()) {
+                throw std::runtime_error("Excluding unrecoverable broken polygon.");
+            }
         }
     }
     geom->normalize(); // Fix direction of ring
@@ -491,25 +497,33 @@ geometry_builder::pg_geoms_t geometry_builder::build_polygons(const multinodelis
             if ((toplevelpolygons > 1) && enable_multi)
             {
                 geom_ptr multipoly(gf.createMultiPolygon(polygons.release()));
-                if (!multipoly->isValid() && !excludepoly) {
-                    multipoly = geom_ptr(multipoly->buffer(0));
-                }
-                multipoly->normalize();
 
-                if ((excludepoly == 0) || (multipoly->isValid())) {
-                    wkbs.emplace_back(multipoly.get(), true, projection);
+                if (!multipoly->isEmpty()) {
+                    if (!multipoly->isValid() && !excludepoly) {
+                        multipoly = geom_ptr(multipoly->buffer(0));
+                        multipoly->normalize();
+                        if (!multipoly->isEmpty() && multipoly->isValid()) {
+                            wkbs.emplace_back(multipoly.get(), true, projection);
+                        }
+                    } else {
+                        multipoly->normalize();
+                        wkbs.emplace_back(multipoly.get(), true, projection);
+                    }
                 }
-            }
-            else
-            {
+            } else {
                 for(unsigned i=0; i<toplevelpolygons; i++) {
                     geom_ptr poly(polygons->at(i));
-                    if (!poly->isValid() && !excludepoly) {
-                        poly = geom_ptr(poly->buffer(0));
-                        poly->normalize();
-                    }
-                    if ((excludepoly == 0) || (poly->isValid())) {
-                        wkbs.emplace_back(poly.get(), true, projection);
+                    if (!poly->isEmpty()) {
+                        if (!poly->isValid() && !excludepoly) {
+                            poly = geom_ptr(poly->buffer(0));
+                            poly->normalize();
+                            if (!poly->isEmpty() && poly->isValid()) {
+                                wkbs.emplace_back(poly.get(), true, projection);
+                            }
+                        } else {
+                            poly->normalize();
+                            wkbs.emplace_back(poly.get(), true, projection);
+                        }
                     }
                 }
             }
@@ -692,7 +706,7 @@ geometry_builder::pg_geoms_t geometry_builder::build_both(const multinodelist_t
                 }
                 multipoly->normalize();
 
-                if ((excludepoly == 0) || (multipoly->isValid())) {
+                if (!multipoly->isEmpty() && multipoly->isValid()) {
                     wkbs.emplace_back(multipoly.get(), true, projection);
                 }
             }
@@ -705,7 +719,7 @@ geometry_builder::pg_geoms_t geometry_builder::build_both(const multinodelist_t
                         poly = geom_ptr(poly->buffer(0));
                         poly->normalize();
                     }
-                    if (!excludepoly || (poly->isValid())) {
+                    if (!poly->isEmpty() && poly->isValid()) {
                         wkbs.emplace_back(poly.get(), true, projection);
                     }
                 }
