@@ -43,6 +43,7 @@ namespace
         {"style",    1, 0, 'S'},
         {"expire-tiles", 1, 0, 'e'},
         {"expire-output", 1, 0, 'o'},
+        {"expire-bbox-size", 1, 0, 214},
         {"output",   1, 0, 'O'},
         {"extra-attributes", 0, 0, 'x'},
         {"hstore", 0, 0, 'k'},
@@ -146,7 +147,7 @@ namespace
                         dense: caching strategy optimised for full planet import\n\
                         chunk: caching strategy optimised for non-contiguous \n\
                             memory allocation\n\
-                        sparse: caching strategy optimised for small extracts\n\
+                        sparse: caching strategy optimised for small imports\n\
                         optimized: automatically combines dense and sparse \n\
                             strategies for optimal storage efficiency. This may\n\
                             us twice as much virtual memory, but no more physical \n\
@@ -168,6 +169,8 @@ namespace
     Expiry options:\n\
        -e|--expire-tiles [min_zoom-]max_zoom    Create a tile expiry list.\n\
        -o|--expire-output filename  Output filename for expired tiles list.\n\
+          --expire-bbox-size Max size for a polygon to expire the whole polygon,\n\
+                             not just the boundary.\n\
     \n\
     Other options:\n\
        -b|--bbox        Apply a bounding box filter on the imported data\n\
@@ -201,7 +204,7 @@ namespace
        -K|--keep-coastlines Keep coastline data rather than filtering it out.\n\
                         By default natural=coastline tagged data will be discarded\n\
                         because renderers usually have shape files for them.\n\
-          --exclude-invalid-polygon   do not import polygons with invalid geometries.\n\
+          --exclude-invalid-polygon   do not attempt to recover invalid geometries.\n\
           --reproject-area   compute area column using spherical mercator coordinates.\n\
        -h|--help        Help information.\n\
        -v|--verbose     Verbose output.\n");
@@ -218,8 +221,8 @@ namespace
             printf("    <flat nodes> is a location where a 19GB file can be saved.\n");
             printf("\n");
             printf("A typical command to update a database imported with the above command is\n");
-            printf("    osmosis --rri workingDirectory=<osmosis dir> --simc --wx - \\\n");
-            printf("      | %s -a -d gis --slim -k --flat-nodes <flat nodes> \n", name);
+            printf("    osmosis --rri workingDirectory=<osmosis dir> --simc --wxc - \\\n");
+            printf("      | %s -a -d gis --slim -k --flat-nodes <flat nodes> -r xml -\n", name);
             printf("where\n");
             printf("    <flat nodes> is the same location as above.\n");
             printf("    <osmosis dir> is the location osmosis replication was initialized to.\n");
@@ -262,7 +265,8 @@ std::string database_options_t::conninfo() const
 options_t::options_t():
     prefix("planet_osm"), scale(DEFAULT_SCALE), projection(reprojection::create_projection(PROJ_SPHERE_MERC)), append(false), slim(false),
     cache(800), tblsmain_index(boost::none), tblsslim_index(boost::none), tblsmain_data(boost::none), tblsslim_data(boost::none), style(OSM2PGSQL_DATADIR "/default.style"),
-    expire_tiles_zoom(-1), expire_tiles_zoom_min(-1), expire_tiles_filename("dirty_tiles"), hstore_mode(HSTORE_NONE), enable_hstore_index(false),
+    expire_tiles_zoom(-1), expire_tiles_zoom_min(-1), expire_tiles_max_bbox(20000.0), expire_tiles_filename("dirty_tiles"),
+    hstore_mode(HSTORE_NONE), enable_hstore_index(false),
     enable_multi(false), hstore_columns(), keep_coastlines(false), parallel_indexing(true),
     #ifdef __amd64__
     alloc_chunkwise(ALLOC_SPARSE | ALLOC_DENSE),
@@ -377,6 +381,9 @@ options_t::options_t(int argc, char *argv[]): options_t()
         case 'o':
             expire_tiles_filename = optarg;
             break;
+        case 214:
+            expire_tiles_max_bbox = atof(optarg);
+            break;
         case 'O':
             output_backend = optarg;
             break;
