@@ -67,7 +67,7 @@ int output_pgsql_t::pgsql_out_node(osmid_t id, const taglist_t &tags, double nod
     if (m_tagtransform->filter_node_tags(tags, *m_export_list.get(), outtags))
         return 1;
 
-    expire->from_bbox(node_lon, node_lat, node_lon, node_lat);
+    expire.from_bbox(node_lon, node_lat, node_lon, node_lat);
     m_tables[t_point]->write_node(id, outtags, node_lat, node_lon);
 
     return 0;
@@ -98,14 +98,14 @@ int output_pgsql_t::pgsql_out_way(osmid_t id, taglist_t &outtags,
     for (const auto& wkb: wkbs) {
         /* FIXME: there should be a better way to detect polygons */
         if (wkb.is_polygon()) {
-            expire->from_nodes_poly(nodes, id);
+            expire.from_nodes_poly(nodes, id);
             if ((wkb.area > 0.0) && m_enable_way_area) {
                 snprintf(tmp, sizeof(tmp), "%g", wkb.area);
                 outtags.push_override(tag_t("way_area", tmp));
             }
             m_tables[t_poly]->write_row(id, outtags, wkb.geom);
         } else {
-            expire->from_nodes_line(nodes);
+            expire.from_nodes_line(nodes);
             m_tables[t_line]->write_row(id, outtags, wkb.geom);
             if (roads)
                 m_tables[t_roads]->write_row(id, outtags, wkb.geom);
@@ -154,7 +154,7 @@ int output_pgsql_t::pgsql_out_relation(osmid_t id, const taglist_t &rel_tags,
 
     char tmp[32];
     for (const auto& wkb: wkbs) {
-        expire->from_wkb(wkb.geom.c_str(), -id);
+        expire.from_wkb(wkb.geom.c_str(), -id);
         /* FIXME: there should be a better way to detect polygons */
         if (wkb.is_polygon()) {
             if ((wkb.area > 0.0) && m_enable_way_area) {
@@ -190,7 +190,7 @@ int output_pgsql_t::pgsql_out_relation(osmid_t id, const taglist_t &rel_tags,
     if (make_boundary) {
         wkbs = builder.build_polygons(xnodes, m_options.enable_multi, id);
         for (const auto& wkb: wkbs) {
-            expire->from_wkb(wkb.geom.c_str(), -id);
+            expire.from_wkb(wkb.geom.c_str(), -id);
             if ((wkb.area > 0.0) && m_enable_way_area) {
                 snprintf(tmp, sizeof(tmp), "%g", wkb.area);
                 outtags.push_override(tag_t("way_area", tmp));
@@ -204,7 +204,7 @@ int output_pgsql_t::pgsql_out_relation(osmid_t id, const taglist_t &rel_tags,
 
 
 void output_pgsql_t::enqueue_ways(pending_queue_t &job_queue, osmid_t id, size_t output_id, size_t& added) {
-    osmid_t const prev = ways_pending_tracker->last_returned();
+    osmid_t const prev = ways_pending_tracker.last_returned();
     if (id_tracker::is_valid(prev) && prev >= id) {
         if (prev > id) {
             job_queue.push(pending_job_t(id, output_id));
@@ -220,7 +220,7 @@ void output_pgsql_t::enqueue_ways(pending_queue_t &job_queue, osmid_t id, size_t
     }
 
     //grab the first one or bail if its not valid
-    osmid_t popped = ways_pending_tracker->pop_mark();
+    osmid_t popped = ways_pending_tracker.pop_mark();
     if(!id_tracker::is_valid(popped))
         return;
 
@@ -230,7 +230,7 @@ void output_pgsql_t::enqueue_ways(pending_queue_t &job_queue, osmid_t id, size_t
             job_queue.push(pending_job_t(popped, output_id));
             added++;
         }
-        popped = ways_pending_tracker->pop_mark();
+        popped = ways_pending_tracker.pop_mark();
     }
 
     //make sure to get this one as well and move to the next
@@ -256,7 +256,7 @@ int output_pgsql_t::pending_way(osmid_t id, int exists) {
             // osmdata_t.
             const idlist_t rel_ids = m_mid->relations_using_way(id);
             for (auto &mid: rel_ids) {
-                rels_pending_tracker->mark(mid);
+                rels_pending_tracker.mark(mid);
             }
         }
 
@@ -273,7 +273,7 @@ int output_pgsql_t::pending_way(osmid_t id, int exists) {
 }
 
 void output_pgsql_t::enqueue_relations(pending_queue_t &job_queue, osmid_t id, size_t output_id, size_t& added) {
-    osmid_t const prev = rels_pending_tracker->last_returned();
+    osmid_t const prev = rels_pending_tracker.last_returned();
     if (id_tracker::is_valid(prev) && prev >= id) {
         if (prev > id) {
             job_queue.push(pending_job_t(id, output_id));
@@ -289,7 +289,7 @@ void output_pgsql_t::enqueue_relations(pending_queue_t &job_queue, osmid_t id, s
     }
 
     //grab the first one or bail if its not valid
-    osmid_t popped = rels_pending_tracker->pop_mark();
+    osmid_t popped = rels_pending_tracker.pop_mark();
     if(!id_tracker::is_valid(popped))
         return;
 
@@ -297,7 +297,7 @@ void output_pgsql_t::enqueue_relations(pending_queue_t &job_queue, osmid_t id, s
     while (popped < id) {
         job_queue.push(pending_job_t(popped, output_id));
         added++;
-        popped = rels_pending_tracker->pop_mark();
+        popped = rels_pending_tracker.pop_mark();
     }
 
     //make sure to get this one as well and move to the next
@@ -352,8 +352,10 @@ void output_pgsql_t::stop()
       }
     }
 
-    expire->output_and_destroy();
-    expire.reset();
+    if (m_options.expire_tiles_zoom_min >= 0) {
+        expire.output_and_destroy(m_options.expire_tiles_filename.c_str(),
+                                  m_options.expire_tiles_zoom_min);
+    }
 }
 
 int output_pgsql_t::node_add(osmid_t id, double lat, double lon, const taglist_t &tags)
@@ -375,7 +377,7 @@ int output_pgsql_t::way_add(osmid_t id, const idlist_t &nds, const taglist_t &ta
 
   /* If this isn't a polygon then it can not be part of a multipolygon
      Hence only polygons are "pending" */
-  if (!filter && polygon) { ways_pending_tracker->mark(id); }
+  if (!filter && polygon) { ways_pending_tracker.mark(id); }
 
   if( !polygon && !filter )
   {
@@ -470,7 +472,7 @@ int output_pgsql_t::node_delete(osmid_t osm_id)
         util::exit_nicely();
     }
 
-    if ( expire->from_db(m_tables[t_point].get(), osm_id) != 0)
+    if ( expire.from_db(m_tables[t_point].get(), osm_id) != 0)
         m_tables[t_point]->delete_row(osm_id);
 
     return 0;
@@ -487,9 +489,9 @@ int output_pgsql_t::pgsql_delete_way_from_output(osmid_t osm_id)
         return 0;
 
     m_tables[t_roads]->delete_row(osm_id);
-    if ( expire->from_db(m_tables[t_line].get(), osm_id) != 0)
+    if ( expire.from_db(m_tables[t_line].get(), osm_id) != 0)
         m_tables[t_line]->delete_row(osm_id);
-    if ( expire->from_db(m_tables[t_poly].get(), osm_id) != 0)
+    if ( expire.from_db(m_tables[t_poly].get(), osm_id) != 0)
         m_tables[t_poly]->delete_row(osm_id);
     return 0;
 }
@@ -509,9 +511,9 @@ int output_pgsql_t::way_delete(osmid_t osm_id)
 int output_pgsql_t::pgsql_delete_relation_from_output(osmid_t osm_id)
 {
     m_tables[t_roads]->delete_row(-osm_id);
-    if ( expire->from_db(m_tables[t_line].get(), -osm_id) != 0)
+    if ( expire.from_db(m_tables[t_line].get(), -osm_id) != 0)
         m_tables[t_line]->delete_row(-osm_id);
-    if ( expire->from_db(m_tables[t_poly].get(), -osm_id) != 0)
+    if ( expire.from_db(m_tables[t_poly].get(), -osm_id) != 0)
         m_tables[t_poly]->delete_row(-osm_id);
     return 0;
 }
@@ -578,21 +580,18 @@ int output_pgsql_t::start()
     return 0;
 }
 
-std::shared_ptr<output_t> output_pgsql_t::clone(const middle_query_t* cloned_middle) const {
-    output_pgsql_t *clone = new output_pgsql_t(*this);
+std::shared_ptr<output_t> output_pgsql_t::clone(const middle_query_t* cloned_middle) const
+{
+    auto *clone = new output_pgsql_t(*this);
     clone->m_mid = cloned_middle;
-    //NOTE: we need to know which ways were used by relations so each thread
-    //must have a copy of the original marked done ways, its read only so its ok
-    clone->ways_done_tracker = ways_done_tracker;
     return std::shared_ptr<output_t>(clone);
 }
 
-output_pgsql_t::output_pgsql_t(const middle_query_t* mid_, const options_t &options_)
-    : output_t(mid_, options_),
-      ways_pending_tracker(new id_tracker()),
-      ways_done_tracker(new id_tracker()),
-      rels_pending_tracker(new id_tracker()) {
-
+output_pgsql_t::output_pgsql_t(const middle_query_t* mid, const options_t &o)
+    : output_t(mid, o),
+      expire(o.expire_tiles_zoom, o.expire_tiles_max_bbox, o.projection),
+      ways_done_tracker(new id_tracker())
+{
     reproj = m_options.projection;
     builder.set_exclude_broken_polygon(m_options.excludepoly);
     if (m_options.reproject_area) builder.set_reprojection(reproj.get());
@@ -610,8 +609,6 @@ output_pgsql_t::output_pgsql_t(const middle_query_t* mid_, const options_t &opti
         util::exit_nicely();
     }
 
-    expire.reset(new expire_tiles(&m_options));
-
     //for each table
     m_tables.reserve(t_MAX);
     for (int i = 0; i < t_MAX; i++) {
@@ -661,9 +658,13 @@ output_pgsql_t::output_pgsql_t(const middle_query_t* mid_, const options_t &opti
 
 output_pgsql_t::output_pgsql_t(const output_pgsql_t& other):
     output_t(other.m_mid, other.m_options), m_tagtransform(new tagtransform(&m_options)), m_enable_way_area(other.m_enable_way_area),
-    m_export_list(new export_list(*other.m_export_list)), reproj(other.reproj),
-    expire(new expire_tiles(&m_options)),
-    ways_pending_tracker(new id_tracker()), ways_done_tracker(new id_tracker()), rels_pending_tracker(new id_tracker())
+    m_export_list(new export_list(*other.m_export_list)),
+    expire(m_options.expire_tiles_zoom, m_options.expire_tiles_max_bbox,
+           m_options.projection),
+    reproj(other.reproj),
+    //NOTE: we need to know which ways were used by relations so each thread
+    //must have a copy of the original marked done ways, its read only so its ok
+    ways_done_tracker(other.ways_done_tracker)
 {
     builder.set_exclude_broken_polygon(m_options.excludepoly);
     if (m_options.reproject_area) builder.set_reprojection(reproj.get());
@@ -677,24 +678,23 @@ output_pgsql_t::~output_pgsql_t() {
 }
 
 size_t output_pgsql_t::pending_count() const {
-    return ways_pending_tracker->size() + rels_pending_tracker->size();
+    return ways_pending_tracker.size() + rels_pending_tracker.size();
 }
 
-void output_pgsql_t::merge_pending_relations(std::shared_ptr<output_t> other) {
-    std::shared_ptr<id_tracker> tracker = other->get_pending_relations();
-    osmid_t id;
-    while(tracker.get() && id_tracker::is_valid((id = tracker->pop_mark()))){
-        rels_pending_tracker->mark(id);
+void output_pgsql_t::merge_pending_relations(output_t *other)
+{
+    auto opgsql = dynamic_cast<output_pgsql_t *>(other);
+    if (opgsql) {
+        osmid_t id;
+        while (id_tracker::is_valid((id = opgsql->rels_pending_tracker.pop_mark()))) {
+            rels_pending_tracker.mark(id);
+        }
     }
 }
-void output_pgsql_t::merge_expire_trees(std::shared_ptr<output_t> other) {
-    if(other->get_expire_tree().get())
-        expire->merge_and_destroy(*other->get_expire_tree());
+void output_pgsql_t::merge_expire_trees(output_t *other)
+{
+    auto *opgsql = dynamic_cast<output_pgsql_t *>(other);
+    if (opgsql)
+        expire.merge_and_destroy(opgsql->expire);
 }
 
-std::shared_ptr<id_tracker> output_pgsql_t::get_pending_relations() {
-    return rels_pending_tracker;
-}
-std::shared_ptr<expire_tiles> output_pgsql_t::get_expire_tree() {
-    return expire;
-}
