@@ -1,6 +1,7 @@
 #include "table.hpp"
 #include "options.hpp"
 #include "util.hpp"
+#include "taginfo.hpp"
 
 #include <exception>
 #include <algorithm>
@@ -23,7 +24,7 @@ table_t::table_t(const string& conninfo, const string& name, const string& type,
     columns(columns), hstore_columns(hstore_columns), table_space(table_space), table_space_index(table_space_index)
 {
     //if we dont have any columns
-    if(columns.size() == 0)
+    if(columns.size() == 0 && hstore_mode != HSTORE_ALL)
         throw std::runtime_error((fmt("No columns provided for table %1%") % name).str());
 
     //nothing to copy to start with
@@ -120,22 +121,23 @@ void table_t::start()
     if (!append)
     {
         //define the new table
-        string sql = (fmt("CREATE TABLE %1% (osm_id %2%,") % name % POSTGRES_OSMID_TYPE).str();
+        string sql = (fmt("CREATE UNLOGGED TABLE %1% (osm_id %2%,") % name % POSTGRES_OSMID_TYPE).str();
 
         //first with the regular columns
-        for(columns_t::const_iterator column = columns.begin(); column != columns.end(); ++column)
-            sql += (fmt("\"%1%\" %2%,") % column->first % column->second).str();
+        for (auto const &column : columns) {
+            sql += (fmt("\"%1%\" %2%,") % column.name % column.type_name).str();
+        }
 
         //then with the hstore columns
         for(hstores_t::const_iterator hcolumn = hstore_columns.begin(); hcolumn != hstore_columns.end(); ++hcolumn)
             sql += (fmt("\"%1%\" hstore,") % (*hcolumn)).str();
 
         //add tags column
-        if (hstore_mode != HSTORE_NONE)
-            sql += "\"tags\" hstore)";
-        //or remove the last ", " from the end
-        else
-            sql[sql.length() - 1] = ')';
+        if (hstore_mode != HSTORE_NONE) {
+            sql += "\"tags\" hstore,";
+        }
+
+        sql += (fmt("way geometry(%1%,%2%) )") % type % srid).str();
 
         // The final tables are created with CREATE TABLE AS ... SELECT * FROM ...
         // This means that they won't get this autovacuum setting, so it doesn't
@@ -148,11 +150,7 @@ void table_t::start()
         //create the table
         pgsql_exec_simple(sql_conn, PGRES_COMMAND_OK, sql);
 
-        //add some constraints
-        pgsql_exec_simple(sql_conn, PGRES_TUPLES_OK, (fmt("SELECT AddGeometryColumn('%1%', 'way', %2%, '%3%', 2 )") % name % srid % type).str());
-        pgsql_exec_simple(sql_conn, PGRES_COMMAND_OK, (fmt("ALTER TABLE %1% ALTER COLUMN way SET NOT NULL") % name).str());
-
-        //slim mode needs this to be able to apply diffs
+        //slim mode needs this to be able to delete from tables in pending
         if (slim && !drop_temp) {
             sql = (fmt("CREATE INDEX %1%_pkey ON %1% USING BTREE (osm_id)") % name).str();
             if (table_space_index)
@@ -164,15 +162,13 @@ void table_t::start()
     else {
         //check the columns against those in the existing table
         std::shared_ptr<PGresult> res = pgsql_exec_simple(sql_conn, PGRES_TUPLES_OK, (fmt("SELECT * FROM %1% LIMIT 0") % name).str());
-        for(columns_t::const_iterator column = columns.begin(); column != columns.end(); ++column)
-        {
-            if(PQfnumber(res.get(), ('"' + column->first + '"').c_str()) < 0)
-            {
+        for (auto const &column :  columns) {
+            if (PQfnumber(res.get(), ('"' + column.name + '"').c_str()) < 0) {
 #if 0
                 throw std::runtime_error((fmt("Append failed. Column \"%1%\" is missing from \"%1%\"\n") % info.name).str());
 #else
-                fprintf(stderr, "%s", (fmt("Adding new column \"%1%\" to \"%2%\"\n") % column->first % name).str().c_str());
-                pgsql_exec_simple(sql_conn, PGRES_COMMAND_OK, (fmt("ALTER TABLE %1% ADD COLUMN \"%2%\" %3%") % name % column->first % column->second).str());
+                fprintf(stderr, "%s", (fmt("Adding new column \"%1%\" to \"%2%\"\n") % column.name % name).str().c_str());
+                pgsql_exec_simple(sql_conn, PGRES_COMMAND_OK, (fmt("ALTER TABLE %1% ADD COLUMN \"%2%\" %3%") % name % column.name % column.type_name).str());
 #endif
             }
             //Note: we do not verify the type or delete unused columns
@@ -189,8 +185,9 @@ void table_t::start()
     //generate column list for COPY
     string cols = "osm_id,";
     //first with the regular columns
-    for(columns_t::const_iterator column = columns.begin(); column != columns.end(); ++column)
-        cols += (fmt("\"%1%\",") % column->first).str();
+    for (auto const &column : columns) {
+        cols += (fmt("\"%1%\",") % column.name).str();
+    }
 
     //then with the hstore columns
     for(hstores_t::const_iterator hcolumn = hstore_columns.begin(); hcolumn != hstore_columns.end(); ++hcolumn)
@@ -219,13 +216,9 @@ void table_t::stop()
 
         fprintf(stderr, "Sorting data and creating indexes for %s\n", name.c_str());
 
-        // Special handling for empty geometries because geohash chokes on
-        // empty geometries on postgis 1.5.
-        pgsql_exec_simple(sql_conn, PGRES_COMMAND_OK, (fmt("CREATE TABLE %1%_tmp %2% AS SELECT * FROM %1% ORDER BY CASE WHEN ST_IsEmpty(way) THEN NULL ELSE ST_GeoHash(ST_Transform(ST_Envelope(way),4326),10) END") % name % (table_space ? "TABLESPACE " + table_space.get() : "")).str());
+        pgsql_exec_simple(sql_conn, PGRES_COMMAND_OK, (fmt("CREATE TABLE %1%_tmp %2% AS SELECT * FROM %1% ORDER BY ST_GeoHash(ST_Transform(ST_Envelope(way),4326),10) COLLATE \"C\"") % name % (table_space ? "TABLESPACE " + table_space.get() : "")).str());
         pgsql_exec_simple(sql_conn, PGRES_COMMAND_OK, (fmt("DROP TABLE %1%") % name).str());
         pgsql_exec_simple(sql_conn, PGRES_COMMAND_OK, (fmt("ALTER TABLE %1%_tmp RENAME TO %1%") % name).str());
-        // Re-add constraints if on 1.x. 2.0 has typemod, and they automatically come with CREATE TABLE AS
-        pgsql_exec_simple(sql_conn, PGRES_TUPLES_OK, (fmt("SELECT CASE WHEN PostGIS_Lib_Version() LIKE '1.%%' THEN Populate_Geometry_Columns('%1%'::regclass) ELSE 1 END;") % name).str());
         fprintf(stderr, "Copying %s to cluster by geometry finished\n", name.c_str());
         fprintf(stderr, "Creating geometry index on %s\n", name.c_str());
 
@@ -355,12 +348,10 @@ void table_t::write_row(const osmid_t id, const taglist_t &tags, const std::stri
 void table_t::write_columns(const taglist_t &tags, string& values, std::vector<bool> *used)
 {
     //for each column
-    for(columns_t::const_iterator column = columns.begin(); column != columns.end(); ++column)
-    {
+    for (auto const &column : columns) {
         int idx;
-        if ((idx = tags.indexof(column->first)) >= 0)
-        {
-            escape_type(tags[idx].value, column->second, values);
+        if ((idx = tags.indexof(column.name)) >= 0) {
+            escape_type(tags[idx].value, column.type, values);
             //remember we already used this one so we cant use again later in the hstore column
             if (used)
                 (*used)[idx] = true;
@@ -467,53 +458,60 @@ void table_t::escape4hstore(const char *src, string& dst)
 }
 
 /* Escape data appropriate to the type */
-void table_t::escape_type(const string &value, const string &type, string& dst) {
+void table_t::escape_type(const string &value, ColumnType type, string& dst) {
 
-    // For integers we take the first number, or the average if it's a-b
-    if (type == "int4") {
-        int from, to;
-        int items = sscanf(value.c_str(), "%d-%d", &from, &to);
-        if (items == 1)
-            dst.append((single_fmt % from).str());
-        else if (items == 2)
-            dst.append((single_fmt % ((from + to) / 2)).str());
-        else
-            dst.append("\\N");
-    }
-        /* try to "repair" real values as follows:
-         * assume "," to be a decimal mark which need to be replaced by "."
-         * like int4 take the first number, or the average if it's a-b
-         * assume SI unit (meters)
-         * convert feet to meters (1 foot = 0.3048 meters)
-         * reject anything else
-         */
-    else if (type == "real")
-    {
-        string escaped(value);
-        std::replace(escaped.begin(), escaped.end(), ',', '.');
-
-        float from, to;
-        int items = sscanf(escaped.c_str(), "%f-%f", &from, &to);
-        if (items == 1)
-        {
-            if (escaped.size() > 1 && escaped.substr(escaped.size() - 2).compare("ft") == 0)
-                from *= 0.3048;
-            dst.append((single_fmt % from).str());
-        }
-        else if (items == 2)
-        {
-            if (escaped.size() > 1 && escaped.substr(escaped.size() - 2).compare("ft") == 0)
+    switch (type) {
+        case COLUMN_TYPE_INT:
             {
-                from *= 0.3048;
-                to *= 0.3048;
+                // For integers we take the first number, or the average if it's a-b
+                long from, to;
+                int items = sscanf(value.c_str(), "%ld-%ld", &from, &to);
+                if (items == 1) {
+                    dst.append((single_fmt % from).str());
+                } else if (items == 2) {
+                    dst.append((single_fmt % ((from + to) / 2)).str());
+                } else {
+                    dst.append("\\N");
+                }
+                break;
             }
-            dst.append((single_fmt % ((from + to) / 2)).str());
-        }
-        else
-            dst.append("\\N");
-    }//just a string
-    else
-        escape(value, dst);
+        case COLUMN_TYPE_REAL:
+            /* try to "repair" real values as follows:
+             * assume "," to be a decimal mark which need to be replaced by "."
+             * like int4 take the first number, or the average if it's a-b
+             * assume SI unit (meters)
+             * convert feet to meters (1 foot = 0.3048 meters)
+             * reject anything else
+             */
+            {
+                string escaped(value);
+                std::replace(escaped.begin(), escaped.end(), ',', '.');
+
+                double from, to;
+                int items = sscanf(escaped.c_str(), "%lf-%lf", &from, &to);
+                if (items == 1) {
+                    if (escaped.size() > 1 && escaped.substr(escaped.size() - 2).compare("ft") == 0) {
+                        from *= 0.3048;
+                    }
+                    dst.append((single_fmt % from).str());
+                }
+                else if (items == 2) {
+                    if (escaped.size() > 1 && escaped.substr(escaped.size() - 2).compare("ft") == 0) {
+                        from *= 0.3048;
+                        to *= 0.3048;
+                    }
+                    dst.append((single_fmt % ((from + to) / 2)).str());
+                }
+                else {
+                    dst.append("\\N");
+                }
+                break;
+            }
+        case COLUMN_TYPE_TEXT:
+            //just a string
+            escape(value, dst);
+            break;
+    }
 }
 
 table_t::wkb_reader table_t::get_wkb_reader(const osmid_t id)
